==> wget-1.4.0.tar.gz: wget-1.4.0/src/main.c <==
/* This file is a part of `wget', a utility to retrieve files
   through the WWW.
   
   Copyright (C) 1995,96  Hrvoje Niksic
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_STRING_H
#  include <string.h>
#else
#  include <strings.h>
#endif /* HAVE_STRING_H */
#include <assert.h>
#include <signal.h>

#include "wget.h"
#include "options.h"
#include "main.h"
#include "utils.h"
#include "url.h"
#include "init.h"
#include "retr.h"
#include "recur.h"
#include "ftp.h"
#include "getopt.h"

extern char *version_string;
extern int errno;

struct options opt;

int
main(int argc, char *const *argv)
{
   char **url, **t, *new;
   char *usage = "Usage: %s [options] [URL-list]\n";
   int i, c, nurl, status, append_to_log, first_time, dt;
   const char *exec_name;
   char *com, *val;

   static struct option long_options[] =
   {
      { "backups", no_argument, NULL, 'b' }, /* undocumented! */
      { "continue-ftp", no_argument, NULL, 'c' },
      { "convert-links", no_argument, NULL, 'k' },
      { "debug", no_argument, NULL, 'd' },
      { "email-address", no_argument, NULL, 'E' }, /* undocumented! */
      { "follow-ftp", no_argument, NULL, 'f' },
      { "force-hier", no_argument, NULL, 'x' },
      { "force-html", no_argument, NULL, 'F'},
      { "help", no_argument, NULL, 'h' },
      { "mirror", no_argument, NULL, 'm' },
      { "no-parent", no_argument, NULL, '5' },
      { "prefix-files", no_argument, NULL, 'p' },
      { "quiet", no_argument, NULL, 'q' },
      { "recursive", no_argument, NULL, 'r' },
      { "relative", no_argument, NULL, 'L' },
      { "save-headers", no_argument, NULL, 's' },
      { "server-response", no_argument, NULL, 'S' },
      { "span-hosts", no_argument, NULL, 'H' },
      { "spider", no_argument, NULL, '4' },
      { "timestamping", no_argument, NULL, 'N' },
      { "verbose", no_argument, NULL, 'v' },
      { "version", no_argument, NULL, 'V' },
      
      { "accept", required_argument, NULL, 'A' },
      { "append-output", required_argument, NULL, 'a' },
      { "base", required_argument, NULL, 'B' },
      { "cache", required_argument, NULL, 'C' },
      { "directory-prefix", required_argument, NULL, 'P' },
      { "domains", required_argument, NULL, 'D' },
      { "execute", required_argument, NULL, 'e' },
      { "exclude-directories", required_argument, NULL, 'X' },
      { "glob", required_argument, NULL, 'g' },
      { "header", required_argument, NULL, '3' },
      { "http-passwd", required_argument, NULL, '2' },
      { "http-user", required_argument, NULL, '1' },
      { "input-file", required_argument, NULL, 'i' },
      { "level", required_argument, NULL, 'l' },
      { "no", required_argument, NULL, 'n' },
      { "output-document", required_argument, NULL, 'O' },
      { "output-file", required_argument, NULL, 'o' },
      { "proxy", required_argument, NULL, 'Y' },
      { "quota", required_argument, NULL, 'Q' },
      { "reject", required_argument, NULL, 'R' },
      { "timeout", required_argument, NULL, 'T' },
      { "tries", required_argument, NULL, 't' },
      { "user-agent", required_argument, NULL, 'U' }, /* undocumented! */
      { "use-proxy", required_argument, NULL, 'Y' },
      { 0, 0, 0, 0 }
   };

   append_to_log = 0;
   initialize();

   /* Construct the name of the executable, without the directory
      part. */
   exec_name = strrchr(argv[0], '/');
   if (!exec_name)
      exec_name = argv[0];
   else
      ++exec_name;

   while ((c = getopt_long(argc, argv, "hVpqvdksxmNWrHSLfcFbEY:g:T:U:O:l:n:i:o:a:t:D:A:R:P:B:e:Q:X:",
                           long_options, (int *)0)) != EOF)
   {
      switch(c)
      {
         case 'h':
            printhelp();
            exit(0);
            break;
         case 'V':
            printf("%s\n", version_string);
            exit(0);
            break;
         case 'Y':
            setval("useproxy", optarg);
            break;
         case 'F':
            setval("forcehtml", "on");
            break;
         case 'k':
            setval("convertlinks", "on");
            break;
         case 'g':
            setval("glob", optarg);
            break;
         case 'p':
            setval("prefixfiles", "on");
            break;
         case 'q':
            setval("quiet", "on");
            break;
         case 'v':
            setval("verbose", "on");
            break;
         case 'x':
            setval("dirstruct", "on");
            break;
         case 'm':
            setval("mirror", "on");
            break;
         case 'E':
            printf("%s\n", opt.ftp_pass);
            exit(0);
            break;
         case 'b':
            setval("backups", "on");
            break;
         case 'l':
            setval("reclevel", optarg);
            break;
         case 'd':
            setval("debug", "on");
            break;
         case 's':
            setval("saveheaders", "on");
            break;
         case 'N':
            setval("timestamping", "on");
            break;
         case 'C':
            setval("cache", optarg);
            break;
         case 'U':
            setval("useragent", optarg);
            break;
         case 'O':
            setval("outputdocument", optarg);
            break;
         case '1':
            setval("httpuser", optarg);
            break;
         case '2':
            setval("httppasswd", optarg);
            break;
         case '3':
            setval("header", optarg);
            break;
         case '4':
            setval("spider", "on");
            break;
         case '5':
            setval("noparent", "on");
            break;
         case 'n':
            for (i = 0; i < strlen(optarg); i++)
               switch(optarg[i])
               {
                  case 'v':
                     setval("verbose", "off");
                     break;
                  case 'h':
                     setval("simplehostcheck", "on");
                     break;
                  case 'H':
                     setval("addhostdir", "off");
                     break;
                  case 'd':
                     setval("dirstruct", "off");
                     break;
                  case 'c':
                     setval("noclobber", "on");
                     break;
                  case 'r':
                     setval("removelisting", "off");
                     break;
                  case 'p':
                     setval("noparent", "on");
                     break;
                  default:
                     printf("-n%c option is unknown. Try %s -h.\n", optarg[i],
                            exec_name);
                     exit(1);
               }
            break;
         case 'i':
            setval("input", optarg);
            break;
         case 'r':
            setval("recursive", "on");
            break;
         case 'H':
            setval("spanhosts", "on");
            break;
         case 'S':
            setval("serverresponse", "on");
            break;
         case 'L':
            setval("relativeonly", "on");
            break;
         case 'f':
            setval("followftp", "on");
            break;
         case 'c':
            setval("alwaysrest", "on");
            break;
         case 'o':
            setval("logfile", optarg);
            break;
         case 'a':
            setval("logfile", optarg);
            append_to_log = 1;
            break;
         case 't':
            setval("numtries", optarg);
            break;
         case 'D':
            setval("domains", optarg);
            break;
         case 'A':
            setval("accept", optarg);
            break;
         case 'R':
            setval("reject", optarg);
            break;
         case 'P':
            setval("dirprefix", optarg);
            break;
         case 'T':
            setval("timeout", optarg);
            break;
         case 'B':
            setval("basehref", optarg);
            break;
         case 'e':
            if (parse_line(optarg, &com, &val))
            {
               setval(com, val);
               free(com);
               free(val);
            }
            else
            {
               fprintf(stderr, "%s: invalid command (try --help)\n", optarg);
               exit(1);
            }
            break;
         case 'Q':
            setval("quota", optarg);
            break;
         case 'X':
            setval("excludedirectories", optarg);
            break;
         case '?':
            printf(usage, exec_name);
            exit(0);
            break;
      }
   }
   if (opt.verbose == -1)
      opt.verbose = !opt.quiet;

   /* Sanity checks. */
   if (opt.verbose && opt.quiet)
   {
      printf("Can't be verbose and quiet at the same time.\n");
      printf(usage, exec_name);
      exit(1);
   }
   if (opt.timestamping && opt.noclobber)
   {
      printf("Can't timestamp and not clobber old files at the same time.\n");
      printf(usage, exec_name);
      exit(1);
   }
   nurl = argc - optind;
   if (!nurl && !opt.input_filename)
   {
      /* No URL specified. */
      printhelp();
      exit(1);
   }
   /* Allocate basic pointer. */
   url = (char **)nmalloc((nurl + 1) * sizeof(char *));
   /* Fill in the arguments. */
   for (i = 0; i < nurl; i++, optind++)
      url[i] = nstrdup(argv[optind]);
   url[i] = NULL;
   /* Open log filename. */
   if (opt.lfilename)
   {
      opt.lfile = fopen(opt.lfilename, append_to_log ? "a" : "w");
      if (opt.lfile == NULL)
      {
         perror(opt.lfilename);
         exit(1);
      }
   }
   else
      opt.lfile = stdout;

   /* Change the buffering. */
   setbuf(opt.lfile, NULL);

   /* Open the output filename (if necessary). */
   if (opt.output_document)
   {
      if (ISHYPHEN(opt.output_document))
         opt.dfp = stdout;
      else                      /* not ISHYPHEN(...) */
      {
         opt.dfp = fopen(opt.output_document, "wb");
         if (opt.dfp == NULL)
         {
            perror(opt.output_document);
            exit(1);
         }
      }
   } /* opt.output_document */

   /* Setup the signal handler. */
   signal(SIGHUP, hangup);

   status = RETROK;             /* Initialize it, just-in-case. */
   /* Retrieve the URLs from argument list. */
   for (first_time = 1, t = url; *t; t++)
   {
      char *filename;
      status = retrieve_url(*t, &filename, &new, NULL, &dt);
      if (opt.recursive && status == RETROK && (dt & TEXTHTML))
         status = recursive_retrieve(filename, new ? new : *t,
                                     first_time ? RFIRST_TIME : 0);
      if (new)
         free(new);
      if (filename)
         free(filename);
      first_time = 0;
   }

   /* And then from the input file (if exists). */
   if (opt.input_filename)
   {
      int count;
      status = retrieve_from_file(opt.input_filename, opt.force_html, &count);
      if (!count && !opt.quiet)
         fprintf(opt.lfile, "No URLs found in %s.\n", opt.input_filename);
   }
   /* Print downloaded sum. */
   if (!opt.quiet && (opt.recursive || nurl > 1
                      || (opt.input_filename && opt.downloaded != 0)))
   {
      fprintf(opt.lfile,
              "\nFINISHED --%s--\nDownloaded: %s bytes in %d files\n",
              time_str(NULL), legible(opt.downloaded), opt.numurls);
      /* Print quota warning, if exceeded. */
      if (opt.quota && opt.downloaded > opt.quota)
         fprintf(opt.lfile, "Download quota (%s bytes) EXCEEDED!\n",
                 legible(opt.quota));
   }
   /* Convert all links.  */
   if (opt.convert_links)
   {
      convert_all_links();
   }
   /* Close the log file. */
   fclose(opt.lfile);
   /* Free the local data. */
   free_vec(url);
   /* ...and clean up the global data. */
   cleanup();
   if (status == RETROK)
      return 0;
   else
      return 1;
}


/* Print the help message. */
void
printhelp(void)
{
   printf("          --- %s by Hrvoje Niksic <hniksic@srce.hr> ---\n",
          version_string);
   printf("Options:\n");
   printf("      -o --output-file      log messages to logfile\n");
   printf("      -i --input-file       read URL-s from file (use \"-\" for stdin)\n");
   printf("      -q --quiet            quiet (no output)\n");
   printf("      -t --tries            set number of retries per URL (0 to unlimit)\n");
   printf("      -r --recursive        recursive web-suck -- use with care!\n");
   printf("      -nH --no=H            do not create hostname directories\n");
   printf("      -nv --no=v            turn off verbose messages\n");
   printf("      -l --level            maximum recursion depth (0 to unlimit)\n");
   printf("      -L --relative-only    follow relative links only\n");
   printf("      -D --domains          comma-separated list of domains to accept\n");
   printf("      -A --accept           comma-separated list of extensions to accept\n");
   printf("      -R --reject           comma-separated list of extensions to reject\n");
   printf("      -c --continue-ftp     restart getting an existing file\n\n");
#ifdef NO_RECURSION
   printf("note: recursion disabled!\n\n");
#endif
   printf("Refer to the info documentation for a complete list of options.\n");
   printf("Mail all suggestions and bug reports to hniksic@srce.hr\n\n");
   printf("This program is distributed in the hope that it will be useful,\n");
   printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
   printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
   printf("GNU General Public License for more details.\n");
}

/* Hangup signal handler. When wget receives hangup, it will proceed
   operation as usual, trying to write into a log file. If that is
   impossible, the output will be turned off. */
RETSIGTYPE
hangup(int sig)
{
   char *logname;
   int count;
   
   signal(SIGHUP, hangup);
   if (opt.lfile != stdout)
      return;
   for (count = 0; (logname = unique_name(DEFAULT_LOGFILE, count)) == NULL;
        count++)
      ;
   opt.lfile = fopen(logname, "w");
   if (opt.lfile == NULL)
   {
      printf("%s: %s\n", logname, mystrerror(errno));
      opt.lfile = fopen("/dev/null", "w");
      assert(opt.lfile != NULL);
   }
   /* Change the buffering. */
   setbuf(opt.lfile, NULL);
   printf("SIGHUP received, redirecting output to %s.\n", logname);
   free(logname);
}

==> wget-1.4.1.tar.gz: wget-1.4.1/src/main.c <==
/* This file is a part of `wget', a utility to retrieve files
   through the WWW.
   
   Copyright (C) 1995,96  Hrvoje Niksic
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_STRING_H
#  include <string.h>
#else
#  include <strings.h>
#endif /* HAVE_STRING_H */
#include <assert.h>
#include <signal.h>

#include "wget.h"
#include "options.h"
#include "main.h"
#include "utils.h"
#include "url.h"
#include "init.h"
#include "retr.h"
#include "recur.h"
#include "ftp.h"
#include "getopt.h"

extern char *version_string;
extern int errno;

struct options opt;

int
main(int argc, char *const *argv)
{
   char **url, **t, *new;
   char *usage = "Usage: %s [options] [URL-list]\n";
   int i, c, nurl, status, append_to_log, first_time, dt;
   const char *exec_name;
   char *com, *val;

   static struct option long_options[] =
   {
      { "backups", no_argument, NULL, 'b' }, /* undocumented! */
      { "continue", no_argument, NULL, 'c' },
      { "convert-links", no_argument, NULL, 'k' },
      { "debug", no_argument, NULL, 'd' },
      { "email-address", no_argument, NULL, 'E' }, /* undocumented! */
      { "follow-ftp", no_argument, NULL, 'f' },
      { "force-hier", no_argument, NULL, 'x' },
      { "force-html", no_argument, NULL, 'F'},
      { "help", no_argument, NULL, 'h' },
      { "mirror", no_argument, NULL, 'm' },
      { "no-parent", no_argument, NULL, '5' },
      { "prefix-files", no_argument, NULL, 'p' },
      { "quiet", no_argument, NULL, 'q' },
      { "recursive", no_argument, NULL, 'r' },
      { "relative", no_argument, NULL, 'L' },
      { "save-headers", no_argument, NULL, 's' },
      { "server-response", no_argument, NULL, 'S' },
      { "span-hosts", no_argument, NULL, 'H' },
      { "spider", no_argument, NULL, '4' },
      { "timestamping", no_argument, NULL, 'N' },
      { "verbose", no_argument, NULL, 'v' },
      { "version", no_argument, NULL, 'V' },
      
      { "accept", required_argument, NULL, 'A' },
      { "append-output", required_argument, NULL, 'a' },
      { "base", required_argument, NULL, 'B' },
      { "cache", required_argument, NULL, 'C' },
      { "directory-prefix", required_argument, NULL, 'P' },
      { "domains", required_argument, NULL, 'D' },
      { "dot-style", required_argument, NULL, '6' },
      { "execute", required_argument, NULL, 'e' },
      { "exclude-directories", required_argument, NULL, 'X' },
      { "glob", required_argument, NULL, 'g' },
      { "header", required_argument, NULL, '3' },
      { "htmlify", required_argument, NULL, '7' },
      { "http-passwd", required_argument, NULL, '2' },
      { "http-user", required_argument, NULL, '1' },
      { "include-directories", required_argument, NULL, 'I' },
      { "input-file", required_argument, NULL, 'i' },
      { "level", required_argument, NULL, 'l' },
      { "no", required_argument, NULL, 'n' },
      { "output-document", required_argument, NULL, 'O' },
      { "output-file", required_argument, NULL, 'o' },
      { "proxy", required_argument, NULL, 'Y' },
      { "quota", required_argument, NULL, 'Q' },
      { "reject", required_argument, NULL, 'R' },
      { "timeout", required_argument, NULL, 'T' },
      { "tries", required_argument, NULL, 't' },
      { "user-agent", required_argument, NULL, 'U' }, /* undocumented! */
      { "use-proxy", required_argument, NULL, 'Y' },
      { 0, 0, 0, 0 }
   };

   append_to_log = 0;
   initialize();

   /* Construct the name of the executable, without the directory
      part. */
   exec_name = strrchr(argv[0], '/');
   if (!exec_name)
      exec_name = argv[0];
   else
      ++exec_name;

   while ((c = getopt_long(argc, argv, "hVpqvdksxmNWrHSLfcFbEY:g:T:U:O:l:n:i:o:a:t:D:A:R:P:B:e:Q:X:I:",
                           long_options, (int *)0)) != EOF)
   {
      switch(c)
      {
         case 'h':
            printhelp();
            exit(0);
            break;
         case 'V':
            printf("%s\n", version_string);
            exit(0);
            break;
         case 'Y':
            setval("useproxy", optarg);
            break;
         case 'F':
            setval("forcehtml", "on");
            break;
         case 'k':
            setval("convertlinks", "on");
            break;
         case 'g':
            setval("glob", optarg);
            break;
         case 'p':
            setval("prefixfiles", "on");
            break;
         case 'q':
            setval("quiet", "on");
            break;
         case 'v':
            setval("verbose", "on");
            break;
         case 'x':
            setval("dirstruct", "on");
            break;
         case 'm':
            setval("mirror", "on");
            break;
         case 'E':
            printf("%s\n", opt.ftp_pass);
            exit(0);
            break;
         case 'b':
            setval("backups", "on");
            break;
         case 'l':
            setval("reclevel", optarg);
            break;
         case 'd':
            setval("debug", "on");
            break;
         case 's':
            setval("saveheaders", "on");
            break;
         case 'N':
            setval("timestamping", "on");
            break;
         case 'C':
            setval("cache", optarg);
            break;
         case 'U':
            setval("useragent", optarg);
            break;
         case 'O':
            setval("outputdocument", optarg);
            break;
         case '1':
            setval("httpuser", optarg);
            break;
         case '2':
            setval("httppasswd", optarg);
            break;
         case '3':
            setval("header", optarg);
            break;
         case '4':
            setval("spider", "on");
            break;
         case '5':
            setval("noparent", "on");
            break;
         case '6':
            setval("dotstyle", optarg);
            break;
         case '7':
            setval("htmlify", optarg);
            break;
         case 'n':
            for (i = 0; i < strlen(optarg); i++)
               switch(optarg[i])
               {
                  case 'v':
                     setval("verbose", "off");
                     break;
                  case 'h':
                     setval("simplehostcheck", "on");
                     break;
                  case 'H':
                     setval("addhostdir", "off");
                     break;
                  case 'd':
                     setval("dirstruct", "off");
                     break;
                  case 'c':
                     setval("noclobber", "on");
                     break;
                  case 'r':
                     setval("removelisting", "off");
                     break;
                  case 'p':
                     setval("noparent", "on");
                     break;
                  default:
                     printf("-n%c option is unknown. Try %s -h.\n", optarg[i],
                            exec_name);
                     exit(1);
               }
            break;
         case 'i':
            setval("input", optarg);
            break;
         case 'r':
            setval("recursive", "on");
            break;
         case 'H':
            setval("spanhosts", "on");
            break;
         case 'S':
            setval("serverresponse", "on");
            break;
         case 'L':
            setval("relativeonly", "on");
            break;
         case 'f':
            setval("followftp", "on");
            break;
         case 'c':
            setval("alwaysrest", "on");
            break;
         case 'o':
            setval("logfile", optarg);
            break;
         case 'a':
            setval("logfile", optarg);
            append_to_log = 1;
            break;
         case 't':
            setval("numtries", optarg);
            break;
         case 'D':
            setval("domains", optarg);
            break;
         case 'A':
            setval("accept", optarg);
            break;
         case 'R':
            setval("reject", optarg);
            break;
         case 'P':
            setval("dirprefix", optarg);
            break;
         case 'T':
            setval("timeout", optarg);
            break;
         case 'B':
            setval("basehref", optarg);
            break;
         case 'e':
            if (parse_line(optarg, &com, &val))
            {
               setval(com, val);
               free(com);
               free(val);
            }
            else
            {
               fprintf(stderr, "%s: invalid command (try --help)\n", optarg);
               exit(1);
            }
            break;
         case 'Q':
            setval("quota", optarg);
            break;
         case 'X':
            setval("excludedirectories", optarg);
            break;
         case 'I':
            setval("includedirectories", optarg);
            break;
         case '?':
            printf(usage, exec_name);
            exit(0);
            break;
      }
   }
   if (opt.verbose == -1)
      opt.verbose = !opt.quiet;

   /* Sanity checks. */
   if (opt.verbose && opt.quiet)
   {
      printf("Can't be verbose and quiet at the same time.\n");
      printf(usage, exec_name);
      exit(1);
   }
   if (opt.timestamping && opt.noclobber)
   {
      printf("Can't timestamp and not clobber old files at the same time.\n");
      printf(usage, exec_name);
      exit(1);
   }
   nurl = argc - optind;
   if (!nurl && !opt.input_filename)
   {
      /* No URL specified. */
      printhelp();
      exit(1);
   }
   /* Allocate basic pointer. */
   url = (char **)nmalloc((nurl + 1) * sizeof(char *));
   /* Fill in the arguments. */
   for (i = 0; i < nurl; i++, optind++)
      url[i] = nstrdup(argv[optind]);
   url[i] = NULL;
   /* Open log filename. */
   if (opt.lfilename)
   {
      opt.lfile = fopen(opt.lfilename, append_to_log ? "a" : "w");
      if (opt.lfile == NULL)
      {
         perror(opt.lfilename);
         exit(1);
      }
   }
   else
      opt.lfile = stdout;
   /* Change the buffering. */
   setbuf(opt.lfile, NULL);
#ifdef DEBUG
   if (opt.debug)
      fprintf(opt.lfile, "DEBUG output created by %s.\n-----\n", version_string);
#endif
   /* Open the output filename (if necessary). */
   if (opt.output_document)
   {
      if (ISHYPHEN(opt.output_document))
         opt.dfp = stdout;
      else                      /* not ISHYPHEN(...) */
      {
         opt.dfp = fopen(opt.output_document, "wb");
         if (opt.dfp == NULL)
         {
            perror(opt.output_document);
            exit(1);
         }
      }
   } /* opt.output_document */

   /* Setup the signal handler. */
   signal(SIGHUP, hangup);

   status = RETROK;             /* Initialize it, just-in-case. */
   /* Retrieve the URLs from argument list. */
   for (first_time = 1, t = url; *t; t++)
   {
      char *filename;
      status = retrieve_url(*t, &filename, &new, NULL, &dt);
      if (opt.recursive && status == RETROK && (dt & TEXTHTML))
         status = recursive_retrieve(filename, new ? new : *t,
                                     first_time ? RFIRST_TIME : 0);
      if (new)
         free(new);
      if (filename)
         free(filename);
      first_time = 0;
   }

   /* And then from the input file (if exists). */
   if (opt.input_filename)
   {
      int count;
      status = retrieve_from_file(opt.input_filename, opt.force_html, &count);
      if (!count && !opt.quiet)
         fprintf(opt.lfile, "No URLs found in %s.\n", opt.input_filename);
   }
   /* Print downloaded sum. */
   if (!opt.quiet && (opt.recursive || nurl > 1
                      || (opt.input_filename && opt.downloaded != 0)))
   {
      fprintf(opt.lfile,
              "\nFINISHED --%s--\nDownloaded: %s bytes in %d files\n",
              time_str(NULL), legible(opt.downloaded), opt.numurls);
      /* Print quota warning, if exceeded. */
      if (opt.quota && opt.downloaded > opt.quota)
         fprintf(opt.lfile, "Download quota (%s bytes) EXCEEDED!\n",
                 legible(opt.quota));
   }
   /* Convert all links.  */
   if (opt.convert_links)
   {
      convert_all_links();
   }
   /* Close the log file. */
   fclose(opt.lfile);
   /* Free the local data. */
   free_vec(url);
   /* ...and clean up the global data. */
   cleanup();
   if (status == RETROK)
      return 0;
   else
      return 1;
}


/* Print the help message. */
void
printhelp(void)
{
   printf("          --- %s by Hrvoje Niksic <hniksic@srce.hr> ---\n",
          version_string);
   printf("Options:\n");
   printf("      -o --output-file      log messages to logfile\n");
   printf("      -i --input-file       read URL-s from file (use \"-\" for stdin)\n");
   printf("      -q --quiet            quiet (no output)\n");
   printf("      -t --tries            set number of retries per URL (0 to unlimit)\n");
   printf("      -r --recursive        recursive web-suck -- use with care!\n");
   printf("      -nH --no=H            do not create hostname directories\n");
   printf("      -nv --no=v            turn off verbose messages\n");
   printf("      -l --level            maximum recursion depth (0 to unlimit)\n");
   printf("      -L --relative-only    follow relative links only\n");
   printf("      -D --domains          comma-separated list of domains to accept\n");
   printf("      -A --accept           comma-separated list of extensions to accept\n");
   printf("      -R --reject           comma-separated list of extensions to reject\n");
   printf("      -c --continue-ftp     restart getting an existing file\n\n");
#ifdef NO_RECURSION
   printf("note: recursion disabled!\n\n");
#endif
   printf("Refer to the info documentation for a complete list of options.\n");
   printf("Mail all suggestions and bug reports to hniksic@srce.hr\n\n");
   printf("This program is distributed in the hope that it will be useful,\n");
   printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
   printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
   printf("GNU General Public License for more details.\n");
}

/* Hangup signal handler. When wget receives hangup, it will proceed
   operation as usual, trying to write into a log file. If that is
   impossible, the output will be turned off. */
RETSIGTYPE
hangup(int sig)
{
   char *logname;
   int count;
   
   signal(SIGHUP, hangup);
   if (opt.lfile != stdout)
      return;
   for (count = 0; (logname = unique_name(DEFAULT_LOGFILE, count)) == NULL;
        count++)
      ;
   opt.lfile = fopen(logname, "w");
   if (opt.lfile == NULL)
   {
      printf("%s: %s\n", logname, mystrerror(errno));
      opt.lfile = fopen("/dev/null", "w");
      assert(opt.lfile != NULL);
   }
   /* Change the buffering. */
   setbuf(opt.lfile, NULL);
   printf("SIGHUP received, redirecting output to %s.\n", logname);
   free(logname);
}
